package com.netmera.netmera_flutter_sdk;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.work.Data;
import androidx.work.OneTimeWorkRequest;
import androidx.work.WorkManager;
import androidx.work.WorkRequest;
import androidx.work.Worker;
import androidx.work.WorkerParameters;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;

import io.flutter.FlutterInjector;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.view.FlutterCallbackInformation;
import io.flutter.view.FlutterMain;
import io.flutter.view.FlutterNativeView;
import io.flutter.view.FlutterRunArguments;


/**
 * Copyright (c) 2021 Inomera Research.
 *
 * @author Burak Soykal
 */
public class FNetmeraWorker extends Worker implements MethodChannel.MethodCallHandler {

    private static final String INTENT_KEY = "INTENT_KEY";
    private static final String SERVICE_INITIALIZED = "serviceInitialized";
    private static final AtomicBoolean serviceStarted = new AtomicBoolean(false);
    private MethodChannel backgroundChannel;

    private final Context context;
    private final ArrayDeque<List<Object>> queue = new ArrayDeque<>();


    public FNetmeraWorker(@NonNull Context context, @NonNull WorkerParameters workerParams) {
        super(context, workerParams);
        this.context = context;
    }

    public static void createAndStart(Context context, Intent intent) {
        Data inputData = new Data.Builder()
                .putAll(bundleToHashMap(intent.getExtras()))
                .putString(INTENT_KEY, intent.getAction())
                .build();
        WorkRequest request = new OneTimeWorkRequest.Builder(FNetmeraWorker.class).setInputData(inputData).build();
        WorkManager.getInstance(context).enqueue(request);
    }

    private void startService(Context context) {
        FlutterNativeView backgroundFlutterView = null;
        synchronized (serviceStarted) {
            if (backgroundFlutterView == null) {
                Long callbackHandle = getFromSharedPref(context, NetmeraFlutterSdkPlugin.CALLBACK_DISPATCHER_HANDLE_KEY);
                FlutterInjector.instance().flutterLoader().ensureInitializationComplete(context, null);
                FlutterCallbackInformation callbackInfo = FlutterCallbackInformation.lookupCallbackInformation(callbackHandle);
                if (callbackInfo == null || FNetmera.nmCallback == null) return;
                backgroundFlutterView = new FlutterNativeView(context, true);
                FNetmera.nmCallback.registerWith(backgroundFlutterView.getPluginRegistry());
                FlutterRunArguments args = new FlutterRunArguments();
                args.bundlePath = FlutterInjector.instance().flutterLoader().findAppBundlePath();
                args.entrypoint = callbackInfo.callbackName;
                args.libraryPath = callbackInfo.callbackLibraryPath;
                backgroundFlutterView.runFromBundle(args);
            }
        }

        FlutterInjector.instance().flutterLoader().ensureInitializationComplete(context, null);
        backgroundChannel = new MethodChannel(backgroundFlutterView, "flutter_nm_sdk_background");
        backgroundChannel.setMethodCallHandler(this);
    }

    @NonNull
    @Override
    public Result doWork() {
        final String action = getInputData().getString(INTENT_KEY);
        final HashMap<String, Object> bundle = new HashMap<>();

        for (Map.Entry<String, Object> entry : getInputData().getKeyValueMap().entrySet()) {
            if (!entry.getKey().equals(INTENT_KEY)) {
                bundle.put(entry.getKey(), entry.getValue());
            }
        }
        new Handler(Looper.getMainLooper()).post(
                new Runnable() {
                    @Override
                    public void run() {
                        startService(context);
                        ArrayList<Object> list = new ArrayList<>();
                        list.add(getFromSharedPref(context, action));
                        list.add(bundle);

                        synchronized (serviceStarted) {
                            if (!serviceStarted.get()) {
                                queue.add(list);
                            } else {
                                backgroundChannel.invokeMethod("", list);
                            }
                        }
                    }
                });
        return Result.success();
    }

    @Override
    public void onMethodCall(@NonNull final MethodCall call, @NonNull final MethodChannel.Result result) {
        new Handler(Looper.getMainLooper()).post(
                new Runnable() {
                    @Override
                    public void run() {
                        if (SERVICE_INITIALIZED.equals(call.method)) {
                            synchronized (serviceStarted) {
                                while (!queue.isEmpty()) {
                                    backgroundChannel.invokeMethod("", queue.remove());
                                }
                                serviceStarted.set(true);
                            }
                        }
                        result.success(null);
                    }
                });
    }

    private static Long getFromSharedPref(Context context, String key) {
        return context.getSharedPreferences(
                NetmeraFlutterSdkPlugin.SHARED_PREFERENCES_KEY,
                Context.MODE_PRIVATE)
                .getLong(key, 0);
    }

    private static HashMap<String, Object> bundleToHashMap(@Nullable Bundle bundle) {
        HashMap<String, Object> map = new HashMap<>();
        if (bundle == null) {
            return map;
        }
        Set<String> keys = bundle.keySet();
        for (String key : keys) {
            map.put(key, bundle.get(key));
        }
        return map;
    }
}
